/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.12.0
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // fFyWQ6CvV9Me_FkwWAL_DwxI_VQROw8tyzSp5_zI8_4.jwk
  0x0,0x0,0x0,0xdc,
  0x7b,
  0x20,0x22,0x61,0x6c,0x67,0x22,0x3a,0x20,0x22,0x45,0x53,0x32,0x35,0x36,0x22,0x2c,
  0x20,0x22,0x6b,0x74,0x79,0x22,0x3a,0x20,0x22,0x45,0x43,0x22,0x2c,0x20,0x22,0x63,
  0x72,0x76,0x22,0x3a,0x20,0x22,0x50,0x2d,0x32,0x35,0x36,0x22,0x2c,0x20,0x22,0x75,
  0x73,0x65,0x22,0x3a,0x20,0x22,0x73,0x69,0x67,0x22,0x2c,0x20,0x22,0x6b,0x69,0x64,
  0x22,0x3a,0x20,0x22,0x66,0x46,0x79,0x57,0x51,0x36,0x43,0x76,0x56,0x39,0x4d,0x65,
  0x5f,0x46,0x6b,0x77,0x57,0x41,0x4c,0x5f,0x44,0x77,0x78,0x49,0x5f,0x56,0x51,0x52,
  0x4f,0x77,0x38,0x74,0x79,0x7a,0x53,0x70,0x35,0x5f,0x7a,0x49,0x38,0x5f,0x34,0x22,
  0x2c,0x20,0x22,0x78,0x22,0x3a,0x20,0x22,0x58,0x53,0x78,0x75,0x77,0x57,0x5f,0x56,
  0x49,0x5f,0x73,0x36,0x6c,0x41,0x77,0x36,0x4c,0x41,0x6c,0x4c,0x38,0x4e,0x37,0x52,
  0x45,0x47,0x7a,0x51,0x64,0x5f,0x7a,0x58,0x65,0x49,0x56,0x44,0x48,0x50,0x5f,0x6a,
  0x5f,0x44,0x6f,0x22,0x2c,0x20,0x22,0x79,0x22,0x3a,0x20,0x22,0x38,0x38,0x2d,0x61,
  0x49,0x34,0x57,0x41,0x45,0x6c,0x34,0x59,0x6d,0x55,0x70,0x65,0x77,0x34,0x30,0x61,
  0x39,0x76,0x71,0x5f,0x77,0x35,0x4f,0x63,0x46,0x76,0x73,0x75,0x61,0x4b,0x4d,0x78,
  0x4a,0x52,0x4c,0x52,0x4c,0x4c,0x30,0x22,0x20,0x7d,0xa,
  
};

static const unsigned char qt_resource_name[] = {
  // org.kde.khealthcertificate
  0x0,0x1a,
  0xb,0x4e,0x8d,0xa5,
  0x0,0x6f,
  0x0,0x72,0x0,0x67,0x0,0x2e,0x0,0x6b,0x0,0x64,0x0,0x65,0x0,0x2e,0x0,0x6b,0x0,0x68,0x0,0x65,0x0,0x61,0x0,0x6c,0x0,0x74,0x0,0x68,0x0,0x63,0x0,0x65,
  0x0,0x72,0x0,0x74,0x0,0x69,0x0,0x66,0x0,0x69,0x0,0x63,0x0,0x61,0x0,0x74,0x0,0x65,
    // shc
  0x0,0x3,
  0x0,0x0,0x79,0xe3,
  0x0,0x73,
  0x0,0x68,0x0,0x63,
    // certs
  0x0,0x5,
  0x0,0x69,0xc9,0xb3,
  0x0,0x63,
  0x0,0x65,0x0,0x72,0x0,0x74,0x0,0x73,
    // fFyWQ6CvV9Me_FkwWAL_DwxI_VQROw8tyzSp5_zI8_4.jwk
  0x0,0x2f,
  0xc,0x95,0xc1,0x7b,
  0x0,0x66,
  0x0,0x46,0x0,0x79,0x0,0x57,0x0,0x51,0x0,0x36,0x0,0x43,0x0,0x76,0x0,0x56,0x0,0x39,0x0,0x4d,0x0,0x65,0x0,0x5f,0x0,0x46,0x0,0x6b,0x0,0x77,0x0,0x57,
  0x0,0x41,0x0,0x4c,0x0,0x5f,0x0,0x44,0x0,0x77,0x0,0x78,0x0,0x49,0x0,0x5f,0x0,0x56,0x0,0x51,0x0,0x52,0x0,0x4f,0x0,0x77,0x0,0x38,0x0,0x74,0x0,0x79,
  0x0,0x7a,0x0,0x53,0x0,0x70,0x0,0x35,0x0,0x5f,0x0,0x7a,0x0,0x49,0x0,0x38,0x0,0x5f,0x0,0x34,0x0,0x2e,0x0,0x6a,0x0,0x77,0x0,0x6b,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9f,0xb7,0x6e,0x21,0xe0,
  // :/org.kde.khealthcertificate
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9f,0xb7,0x6e,0x21,0xe0,
  // :/org.kde.khealthcertificate/shc
  0x0,0x0,0x0,0x3a,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9f,0xb7,0x6e,0x21,0xe0,
  // :/org.kde.khealthcertificate/shc/certs
  0x0,0x0,0x0,0x46,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x4,
0x0,0x0,0x1,0x9f,0xb7,0x6e,0x21,0xe0,
  // :/org.kde.khealthcertificate/shc/certs/fFyWQ6CvV9Me_FkwWAL_DwxI_VQROw8tyzSp5_zI8_4.jwk
  0x0,0x0,0x0,0x56,0x0,0x0,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9f,0xb7,0x6e,0x21,0xe0,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_shc_certs_manual)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_shc_certs_manual)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_shc_certs_manual)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_shc_certs_manual)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_shc_certs_manual)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_shc_certs_manual)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
