/****************************************************************************
** Meta object code from reading C++ file 'khealthcertificate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../khealthcertificate-26.07.90/src/lib/khealthcertificate.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'khealthcertificate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18KHealthCertificateE_t {};
} // unnamed namespace

template <> constexpr inline auto KHealthCertificate::qt_create_metaobjectdata<qt_meta_tag_ZN18KHealthCertificateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KHealthCertificate",
        "CertificateType",
        "Vaccination",
        "Test",
        "Recovery",
        "CertificateValidation",
        "Valid",
        "Partial",
        "Invalid",
        "Unknown",
        "SignatureValidation",
        "ValidSignature",
        "InvalidSignature",
        "UnknownSignature",
        "UncheckedSignature"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CertificateType'
        QtMocHelpers::EnumData<enum CertificateType>(1, 1, QMC::EnumFlags{}).add({
            {    2, CertificateType::Vaccination },
            {    3, CertificateType::Test },
            {    4, CertificateType::Recovery },
        }),
        // enum 'CertificateValidation'
        QtMocHelpers::EnumData<enum CertificateValidation>(5, 5, QMC::EnumFlags{}).add({
            {    6, CertificateValidation::Valid },
            {    7, CertificateValidation::Partial },
            {    8, CertificateValidation::Invalid },
            {    9, CertificateValidation::Unknown },
        }),
        // enum 'SignatureValidation'
        QtMocHelpers::EnumData<enum SignatureValidation>(10, 10, QMC::EnumFlags{}).add({
            {   11, SignatureValidation::ValidSignature },
            {   12, SignatureValidation::InvalidSignature },
            {   13, SignatureValidation::UnknownSignature },
            {   14, SignatureValidation::UncheckedSignature },
        }),
    };
    return QtMocHelpers::metaObjectData<KHealthCertificate, qt_meta_tag_ZN18KHealthCertificateE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KHealthCertificate::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KHealthCertificateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18KHealthCertificateE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18KHealthCertificateE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
