/****************************************************************************
** Meta object code from reading C++ file 'krecoverycertificate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../khealthcertificate-26.07.90/src/lib/krecoverycertificate.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'krecoverycertificate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20KRecoveryCertificateE_t {};
} // unnamed namespace

template <> constexpr inline auto KRecoveryCertificate::qt_create_metaobjectdata<qt_meta_tag_ZN20KRecoveryCertificateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KRecoveryCertificate",
        "type",
        "KHealthCertificate::CertificateType",
        "name",
        "dateOfBirth",
        "dateOfPositiveTest",
        "validFrom",
        "validUntil",
        "disease",
        "certificateIssuer",
        "certificateId",
        "certificateIssueDate",
        "certificateExpiryDate",
        "signatureState",
        "KHealthCertificate::SignatureValidation",
        "validationState",
        "KHealthCertificate::CertificateValidation",
        "rawData"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<KHealthCertificate::CertificateType>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'dateOfBirth'
        QtMocHelpers::PropertyData<QDate>(4, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'dateOfPositiveTest'
        QtMocHelpers::PropertyData<QDate>(5, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validFrom'
        QtMocHelpers::PropertyData<QDate>(6, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'validUntil'
        QtMocHelpers::PropertyData<QDate>(7, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'disease'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'certificateIssuer'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'certificateId'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'certificateIssueDate'
        QtMocHelpers::PropertyData<QDateTime>(11, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'certificateExpiryDate'
        QtMocHelpers::PropertyData<QDateTime>(12, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'signatureState'
        QtMocHelpers::PropertyData<KHealthCertificate::SignatureValidation>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validationState'
        QtMocHelpers::PropertyData<KHealthCertificate::CertificateValidation>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'rawData'
        QtMocHelpers::PropertyData<QByteArray>(17, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KRecoveryCertificate, qt_meta_tag_ZN20KRecoveryCertificateE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN20KRecoveryCertificateE[] = {
    QMetaObject::SuperData::link<KHealthCertificate::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KRecoveryCertificate::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KRecoveryCertificateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KRecoveryCertificateE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN20KRecoveryCertificateE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20KRecoveryCertificateE_t>.metaTypes,
    nullptr
} };

void KRecoveryCertificate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<KRecoveryCertificate *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KHealthCertificate::CertificateType*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<QDate*>(_v) = _t->dateOfBirth(); break;
        case 3: *reinterpret_cast<QDate*>(_v) = _t->dateOfPositiveTest(); break;
        case 4: *reinterpret_cast<QDate*>(_v) = _t->validFrom(); break;
        case 5: *reinterpret_cast<QDate*>(_v) = _t->validUntil(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->disease(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->certificateIssuer(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->certificateId(); break;
        case 9: *reinterpret_cast<QDateTime*>(_v) = _t->certificateIssueDate(); break;
        case 10: *reinterpret_cast<QDateTime*>(_v) = _t->certificateExpiryDate(); break;
        case 11: *reinterpret_cast<KHealthCertificate::SignatureValidation*>(_v) = _t->signatureState(); break;
        case 12: *reinterpret_cast<KHealthCertificate::CertificateValidation*>(_v) = _t->validationState(); break;
        case 13: *reinterpret_cast<QByteArray*>(_v) = _t->rawData(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
