/****************************************************************************
** Meta object code from reading C++ file 'kvaccinationcertificate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../khealthcertificate-26.07.90/src/lib/kvaccinationcertificate.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kvaccinationcertificate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23KVaccinationCertificateE_t {};
} // unnamed namespace

template <> constexpr inline auto KVaccinationCertificate::qt_create_metaobjectdata<qt_meta_tag_ZN23KVaccinationCertificateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KVaccinationCertificate",
        "type",
        "KHealthCertificate::CertificateType",
        "name",
        "dateOfBirth",
        "date",
        "disease",
        "vaccineType",
        "vaccine",
        "vaccineUrl",
        "QUrl",
        "manufacturer",
        "dose",
        "totalDoses",
        "country",
        "certificateIssuer",
        "certificateId",
        "certificateIssueDate",
        "certificateExpiryDate",
        "signatureState",
        "KHealthCertificate::SignatureValidation",
        "validationState",
        "KHealthCertificate::CertificateValidation",
        "rawData",
        "vaccinationState",
        "VaccinationState",
        "Invalid",
        "VaccinationTooRecent",
        "PartiallyVaccinated",
        "Vaccinated",
        "FullyVaccinated"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<KHealthCertificate::CertificateType>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'dateOfBirth'
        QtMocHelpers::PropertyData<QDate>(4, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'date'
        QtMocHelpers::PropertyData<QDate>(5, QMetaType::QDate, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'disease'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'vaccineType'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'vaccine'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'vaccineUrl'
        QtMocHelpers::PropertyData<QUrl>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'manufacturer'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'dose'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'totalDoses'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'country'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'certificateIssuer'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'certificateId'
        QtMocHelpers::PropertyData<QString>(16, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'certificateIssueDate'
        QtMocHelpers::PropertyData<QDateTime>(17, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'certificateExpiryDate'
        QtMocHelpers::PropertyData<QDateTime>(18, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'signatureState'
        QtMocHelpers::PropertyData<KHealthCertificate::SignatureValidation>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'validationState'
        QtMocHelpers::PropertyData<KHealthCertificate::CertificateValidation>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'rawData'
        QtMocHelpers::PropertyData<QByteArray>(23, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'vaccinationState'
        QtMocHelpers::PropertyData<enum VaccinationState>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'VaccinationState'
        QtMocHelpers::EnumData<enum VaccinationState>(25, 25, QMC::EnumFlags{}).add({
            {   26, VaccinationState::Invalid },
            {   27, VaccinationState::VaccinationTooRecent },
            {   28, VaccinationState::PartiallyVaccinated },
            {   29, VaccinationState::Vaccinated },
            {   30, VaccinationState::FullyVaccinated },
        }),
    };
    return QtMocHelpers::metaObjectData<KVaccinationCertificate, qt_meta_tag_ZN23KVaccinationCertificateE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN23KVaccinationCertificateE[] = {
    QMetaObject::SuperData::link<KHealthCertificate::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject KVaccinationCertificate::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KVaccinationCertificateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23KVaccinationCertificateE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN23KVaccinationCertificateE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23KVaccinationCertificateE_t>.metaTypes,
    nullptr
} };

void KVaccinationCertificate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<KVaccinationCertificate *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<KHealthCertificate::CertificateType*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast<QDate*>(_v) = _t->dateOfBirth(); break;
        case 3: *reinterpret_cast<QDate*>(_v) = _t->date(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->disease(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->vaccineType(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->vaccine(); break;
        case 7: *reinterpret_cast<QUrl*>(_v) = _t->vaccineUrl(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->manufacturer(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->dose(); break;
        case 10: *reinterpret_cast<int*>(_v) = _t->totalDoses(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->country(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->certificateIssuer(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->certificateId(); break;
        case 14: *reinterpret_cast<QDateTime*>(_v) = _t->certificateIssueDate(); break;
        case 15: *reinterpret_cast<QDateTime*>(_v) = _t->certificateExpiryDate(); break;
        case 16: *reinterpret_cast<KHealthCertificate::SignatureValidation*>(_v) = _t->signatureState(); break;
        case 17: *reinterpret_cast<KHealthCertificate::CertificateValidation*>(_v) = _t->validationState(); break;
        case 18: *reinterpret_cast<QByteArray*>(_v) = _t->rawData(); break;
        case 19: *reinterpret_cast<enum VaccinationState*>(_v) = _t->vaccinationState(); break;
        default: break;
        }
    }
}
QT_WARNING_POP
